import {
  detectSQLInjection,
  isSQLSentence,
} from '../detectors/sqlInjectionDetector';
import { detectXSS } from '../detectors/xssDetector';
import { detectScriptInjection } from '../detectors/scriptInjectionDetector';

interface ScanOptions {
  keys: boolean;
  values: boolean;
  detectSentences: boolean;
  detectReservedWords: boolean;
}

export function scanBody(body: any, options: ScanOptions): string[] {
  const issues: string[] = [];

  const { keys, values, detectSentences, detectReservedWords } = options;

  if (keys) {
    for (const key of Object.keys(body)) {
      if (detectSentences && isSQLSentence(key)) {
        issues.push(`SQL Sentence detected in body key ${key}`);
      }
      if (detectReservedWords && detectSQLInjection(key)) {
        issues.push(`SQL Injection detected in body key ${key}`);
      }
      if (detectXSS(key)) {
        issues.push(`XSS detected in body key ${key}`);
      }
      if (detectScriptInjection(key)) {
        issues.push(`Script Injection detected in body key ${key}`);
      }
    }
  }

  if (values) {
    for (const [key, value] of Object.entries(body)) {
      const valueString = String(value);
      if (detectSentences && isSQLSentence(valueString)) {
        issues.push(
          `SQL Sentence detected in body value at key ${key}: ${valueString}`,
        );
      }
      if (detectReservedWords && detectSQLInjection(valueString)) {
        issues.push(
          `SQL Injection detected in body value at key ${key}: ${valueString}`,
        );
      }
      if (detectXSS(valueString)) {
        issues.push(`XSS detected in body value at key ${key}: ${valueString}`);
      }
      if (detectScriptInjection(valueString)) {
        issues.push(
          `Script Injection detected in body value at key ${key}: ${valueString}`,
        );
      }
    }
  }

  return issues;
}
